# Copyright (C) 2009, 2010 Martin Pool
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. 
#
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0 
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License. 
#
# Part of Tribunal <https://launchpad.net/tribunal>


"""Main tribunal test app object."""

import optparse
import os.path
import sys

import gtk
import gtk.glade

import tribunal
from tribunal import (
    eventloop,
    util,
    window,
    )
from tribunal.source import (
    FileTestSource,
    NoStreamPresent,
    TestRepositorySource,
    )

    
class ViewSubunitApp(object):
    """Application object. 

    Receives events from the window.
    """
    # the separation of concerns from the Window is: if we later support
    # multiple windows looking at different results, there will be just one
    # app across all of them
    
    def __init__(self):
        self._event_loop = eventloop.GtkLoop()

    def _construct_and_show_window(self):
        glade_filename = util.sibpath(tribunal.__file__, 'ui.glade')
        glade_xml = gtk.glade.XML(glade_filename)
        self._window = window.SubunitWindow(glade_xml)
        glade_xml.signal_autoconnect(self)
        self._set_default_icons()
        self._window.show_all()

    def on_quit_menuitem_activate(self, widget):
        self._event_loop.stop()

    def on_subunit_window_destroy(self, widget):
        self._event_loop.stop()

    def run(self, argv):
        """Run the app: create a window, load the file, handle input.
        
        Return a return code.
        """

        parser = optparse.OptionParser()
        parser.add_option('--testr',
            help='read from a testrepository in the given directory')

        opts, args = parser.parse_args(argv)

        if opts.testr:
            source = TestRepositorySource(opts.testr)
        elif args == ['-']:
            source = FileTestSource(sys.stdin)
        elif len(args) == 1:
            if os.path.isfile(args[0]):
                source = FileTestSource(file(args[0], 'rb'))
            elif os.path.isdir(os.path.join(args[0], ".testrepository")):
                source = TestRepositorySource(".")
            elif os.path.exists(os.path.join(args[0], ".testr.conf")):
                dialog = gtk.Dialog(title="No testrepository present",
                    parent=None,
                    buttons=(gtk.STOCK_YES, gtk.RESPONSE_YES,
                            gtk.STOCK_NO, gtk.RESPONSE_NO))
                label = gtk.Label("No test repository is present. Create one?")
                dialog.vbox.pack_start(label, True, True, 0)
                label.show()
                resp = dialog.run()
                dialog.destroy()
                if resp == gtk.RESPONSE_YES:
                    import testrepository.repository.file
                    factory = testrepository.repository.file.RepositoryFactory()
                    factory.initialise(".")
                    source = TestRepositorySource(".")
            else:
                print "Unable to open %s" % args[0]
                sys.exit(1)
        else:
            source = None

        self._construct_and_show_window()
        if source is not None:
            self._window.set_source(source)
            try:
                stream = source.get_subunit_stream()
            except NoStreamPresent:
                pass
            else:
                self._window.load_from_subunit_stream(stream)
        self._event_loop.start()

    def _set_default_icons(self):
        # XXX: maybe should use pkgresources?
        icon_pixbuf = gtk.gdk.pixbuf_new_from_file(
            os.path.join(os.path.dirname(tribunal.__file__), 'gavel_64x64.png'))
        gtk.window_set_default_icon_list(icon_pixbuf)


