# Copyright (C) 2009, 2010 Martin Pool
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. 
#
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0 
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License. 
#
# Part of Tribunal <https://launchpad.net/tribunal>


"""Stream input into Subunit.
"""


import errno
import sys

import gtk
import glib
import gobject


class InputDriverBase(object):

    def __init__(self, stream, on_finish, protocol_server):
        self.stream = stream
        self.on_finish = on_finish
        self.protocol = protocol_server
        self.paused = False


class FileInputDriver(InputDriverBase):
    """Feeds nonblocking input from a file or pipe to subunit.

    Subunit then passes it on to a TestResult.
    """

    _hup_source_id = _read_source_id = None
    
    def set_paused(self, paused):
        print "set paused: %s" % paused
        if paused:
            self._remove_listeners
        else:
            self.start_reading()
        self.paused = paused

    def start_reading(self):
        self._hup_source_id = gobject.io_add_watch(self.stream,
            gobject.IO_HUP, self.on_hangup)
        self._read_source_id = gobject.io_add_watch(self.stream,
            gobject.IO_IN, self.on_data_available,
            priority=glib.PRIORITY_DEFAULT_IDLE)

    def stop_reading(self):
        self._shutdown()

    def _shutdown(self):
        """Called on end-of-file or connection closed"""
        self._remove_listeners()
        self.protocol.lostConnection()
        if self.on_finish is not None:
            self.on_finish()

    def on_data_available(self, source, condition):
        # XXX: is this guaranteed not to block if there's not a full line
        # available?
        #
        # NB: what we really want here is to read up to about N lines or until
        # the IO would block.  We do want a cap on the amount of IO for any
        # single iteration so that we don't block on local operations which
        # will never block.
        for i in range(100):
            try:
                line = source.readline()
            except IOError, e:
                if e.errno == errno.EWOULDBLOCK:
                    break
                else:
                    raise
            if False:
                print 'read %r, %r' % (source, condition)
                print '    got %r' % line
            if not line:
                self._shutdown()
                sys.stderr.write('done\n')
                return False
            self.protocol.lineReceived(line)
        sys.stderr.write('.')
        return not self.paused      # call me again please

    def on_hangup(self, source, condition):
        self._shutdown()
        return False
        
    def _remove_listeners(self):
        for source_id in (self._hup_source_id, self._read_source_id):
            if source_id is not None:
                gobject.source_remove(source_id)


class InProcessInputDriver(InputDriverBase):
    """Feed from an in-process file-like into subunit.

    This inserts some gaps between reading so that the gtk event loop still
    gets a chance to run.

    In particular this handles reading from testrepository which passes back a
    StringIO, so we should be able to access it all with no physical IO, but
    it may take some time to parse it.
    """
    
    def set_paused(self, paused):
        pass  # not supported; everything is read immediately

    def start_reading(self):
        self._read_everything()

    def _read_everything(self):
        while True:
            # TODO: let the gtk loop run a bit
            line = self.stream.readline()
            print 'read %r' % line
            if not line:
                self._shutdown()
                break
            self.protocol.lineReceived(line)

    def _shutdown(self):
        self.protocol.lostConnection()
        if self.on_finish is not None:
            self.on_finish()
