# Copyright 2008 Jonathan Lange
# Copyright (C) 2009, 2010 Martin Pool
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. 
#
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0 
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License. 
#
# Part of Tribunal <https://launchpad.net/tribunal>


from collections import defaultdict
import gtk


class TableTestResults(object):
    """A TestResult, fed test outcomes by the TestRunner.

    This one retains some data itself, and puts other parts into a TreeStore
    that drives a tree display.
    
    This only deals with the data store, not at all with the view.
    
    The GtkTreeStore has two columns: the test id, and the outcome.

    This is a partially abstract base class; it needs some extra definitions
    of how the tests may into rows.

    :ivar store: gtk TreeStore
    """

    UNKNOWN = 'unknown'
    PASSED = 'passed'
    FAILED = 'failed'
    SKIPPED = 'skipped'
    XFAIL = 'xfail'
    
    COL_OUTCOME = 1
    COL_TEST_ID = 0
    COL_WHY = 2

    def __init__(self, observer):
        """Construct new TribunalTestResult.
        
        :param observer: object to be notified when tests start and 
            finish.
        """
        self.store = gtk.TreeStore(str, str, str)
        self._test_id_to_path = {}
        self._path_to_test_id = {}
        self._details_by_id = {}
        self._error_by_id = {}
        self._counts_by_status = defaultdict(lambda: 0)
        self.observer = observer

    def _clear(self):
        """Completely reset all information"""
        self.store.clear()
        self._test_id_to_path.clear()
        self._path_to_test_id.clear()
        self._details_by_id.clear()
        self._error_by_id.clear()
        self._counts_by_status.clear()

    # from TestResult
    def startTest(self, test):
        test_id = test.id()
        self.observer.on_test_started(test_id)

    # from TestResult
    def addError(self, test, error, details=None):
        self._store_result(test, self.FAILED, error, details)

    # from TestResult
    def addExpectedFailure(self, test, error=None, details=None):
        self._store_result(test, self.XFAIL, error, details)

    # from TestResult
    def addFailure(self, test, error=None, details=None):
        self._store_result(test, self.FAILED, error, details)

    # from TestResult
    def addSkip(self, test, reason=None, details=None):
        self._store_result(test, self.SKIPPED, None, details, skip_reason=reason)

    # from TestResult
    def addSuccess(self, test, details=None):
        self._store_result(test, self.PASSED, None, details)

    def _store_result(self, test, outcome, exc_info, details,
        skip_reason=None):
        """Store a result passed from the test.

        :param outcome: One of the outcome constants, eg SKIPPED
        :param err: Either an exception or a string (in the case of Skip) 
            or None
        :param details: subunit details dictionary or none
        :param test: A test object on which you can at least call .id().
        """
        test_id = test.id()
        new_iter = self.store.append(None, (test_id, outcome, ''))
        path = self.store.get_path(new_iter)
        self._test_id_to_path[test_id] = path
        self._path_to_test_id[path] = test_id
        self._details_by_id[test_id] = details
        if exc_info:
            self._error_by_id[test_id] = '%s: %s' % exc_info[0:2]
        else:
            self._error_by_id[test_id] = skip_reason
        self._counts_by_status[outcome] += 1
        self.observer.on_test_concluded(test_id, outcome)

    # from TestResult
    def stopTest(self, test):
        pass

    def get_detail_text_by_test_id(self, test_id):
        return self._details_by_id[test_id]

    def get_outcome_by_test_id(self, test_id):
        return self.store.get(
            self.store.get_iter(self.get_tree_path(test_id)),
            self.COL_OUTCOME)[0]

    def get_error_by_test_id(self, test_id):
        """Returns a single (probably short) string describing the error 
        or skip."""
        return self._error_by_id[test_id]

    def get_test_id(self, tree_path):
        """Return the fully-qualified test name.

        :param tree_iter: A GTK tree path.
        :return: The fully-qualified Python name of the object, or None if
            there is no test at that path.
        """
        return self._path_to_test_id.get(tree_path, None)

    def get_tree_path(self, test_id):
        return self._test_id_to_path[test_id]
