# Copyright (C) 2009, 2010 Martin Pool
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. 
#
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0 
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License. 
#
# Part of Tribunal <https://launchpad.net/tribunal>



import sys

import gobject
import gtk
import pango
import gtk.glade

from subunit import TestProtocolServer

from tribunal import (
    tree,
    )
from tribunal.inputdriver import (
    FileInputDriver,
    InProcessInputDriver,
    )
from tribunal.source import (
    UnreloadableStream,
    )


def xml_escape(t):
    return t.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;')


def details_dict_to_text(details):
    if not details:
        return ''
    bits = []
    for name, content in sorted(details.items()):
        bits.append(name + ':')
        bits.append(''.join(content.iter_text()))
    return '\n'.join(bits)


class SubunitWindow(object):
    """Per-window application logic.

    Contains references to many gtk controls, in particular

    :ivar subunit_window: the top level gtk window
    :ivar model_filter: TreeModelFilter controlling what's actually visible
        in the list of tests.
    :ivar _outcome_filter: dict from outcome strings (eg 'passed') to bool
        saying whether they should be shown
    """
    
    def __init__(self, glade_xml):
        self.glade = glade_xml
        for i in [
            'result_frame',
            'result_frame_label',
            'result_textview', 
            'show_failed_tb',
            'subunit_window', 
            'statusbar',
            'test_treeview',
            ]:
            w = glade_xml.get_widget(i)
            assert w is not None, \
                "couldn't load %r" % i
            setattr(self, i, w)
        # acts like a TestSuite and feeds into a gtk tree; also sets some
        # parameters on the view
        self.test_result = tree.TableTestResults(self)
        self._connect_tree()
        glade_xml.signal_autoconnect(self)
        self._outcome_filter = dict(
            failed=True,
            skipped=False,
            passed=False,
            xfail=False)
        self._glade_xml = glade_xml
        self._get_widget = glade_xml.get_widget
        # XXX: send misunderstood subunit data to stderr for the sake of
        # debugging; eventually we'd like to show it in a window  -- mbp
        # 20100127
        self._subunit_passthrough = sys.stderr # NullOutputStream()
        self._running_test_statusbar_context = \
            self.statusbar.get_context_id("current test")
        self.source = None

    def _connect_tree(self):
        """Connect the tree model to the view in table view."""
        view = self.test_treeview
        self.model_filter = self.test_result.store.filter_new()
        self.model_filter.set_visible_func(self._row_visible_func)
        view.set_model(self.model_filter)

        view.set_headers_visible(True)
        view.set_headers_clickable(True)
        view.set_show_expanders(False)

        outcome_col = gtk.TreeViewColumn("Outcome",
            gtk.CellRendererText(),
            text=self.test_result.COL_OUTCOME)
        view.append_column(outcome_col)

        test_name_renderer = gtk.CellRendererText()
        test_name_renderer.set_property('ellipsize', pango.ELLIPSIZE_MIDDLE)

        test_name_col = gtk.TreeViewColumn("Test",
            test_name_renderer,
            text=self.test_result.COL_TEST_ID)
        view.append_column(test_name_col)

    def load_from_subunit_stream(self, input_file):
        # feed things from the subunit stream into self.test_result
        #
        # this currently works by running the subunit stream as if it were an
        # actually running test, through the python unittest protocol.  This
        # squashes some of the interesting data, such as folding the
        # attachments into a _StringException.  We probably actually need to
        # load it through a Subunit-specific protocol.
        context = self.statusbar.get_context_id("Loading")
        self.statusbar.push(context, "Loading stream...")
        self.test_result._clear()
        # probably a good guess that the user now wants to look through the
        # test results tree
        self.test_treeview.grab_focus()
        def on_finish():
            self.statusbar.pop(context)

        # dummy_result = testtools.TestResult()

        protocol_server = TestProtocolServer(
            self.test_result,
            self._subunit_passthrough,
            None)
        # if it's a real file, we should do nonblocking callback io
        if getattr(input_file, 'fileno', None):
            driver_class = FileInputDriver
        else:
            driver_class = InProcessInputDriver
        self.input_driver = driver_class(input_file,
            on_finish=on_finish,
            protocol_server=protocol_server)
        self.input_driver.start_reading()

    def on_about_menuitem_activate(self, menuitem):
        aboutdialog = gtk.AboutDialog()
        aboutdialog.set_authors((
            'Jonathan Lange',
            'Martin Pool',
            'Jelmer Vernooij',
            ))
        aboutdialog.set_copyright(u"Copyright \u00a9 2008-2010 Contributors")
        aboutdialog.set_comments(
            "A graphical inspector for Subunit test results")
        aboutdialog.set_license("""
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this program except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software 
distributed under the License is distributed on an "AS IS" BASIS, 
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.""")
        aboutdialog.set_website("http://launchpad.net/tribunal")
        aboutdialog.run()
        aboutdialog.destroy()

    def on_filter_failed_menuitem_toggled(self, menuitem):
        self.set_outcome_filter('failed', menuitem.get_active())

    def on_filter_skipped_menuitem_toggled(self, menuitem):
        self.set_outcome_filter('skipped', menuitem.get_active())

    def on_filter_passed_menuitem_toggled(self, menuitem):
        self.set_outcome_filter('passed', menuitem.get_active())
        
    def on_filter_xfail_menuitem_toggled(self, menuitem):
        self.set_outcome_filter('xfail', menuitem.get_active())
        
    def on_pause_input_tb_toggled(self, toggle):
        self.input_driver.set_paused(toggle.get_active())

    def on_reload_button_clicked(self, reload_button):
        try:
            self.reload_from_source()
        except UnreloadableStream:
            dialog = gtk.Dialog(title="Stream not reloadable",
                parent=self.subunit_window,
                buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
            label = gtk.Label("This stream is not reloadable. Perhaps it is a pipe?")
            dialog.vbox.pack_start(label, True, True, 0)
            label.show()
            dialog.run()
            dialog.destroy()

    def on_show_failed_tb_toggled(self, tb_toggle):
        self.set_outcome_filter('failed', tb_toggle.get_active())

    def on_show_skipped_tb_toggled(self, tb_toggle):
        self.set_outcome_filter('skipped', tb_toggle.get_active())

    def on_show_passed_tb_toggled(self, tb_toggle):
        self.set_outcome_filter('passed', tb_toggle.get_active())
        
    def on_show_xfail_tb_toggled(self, tb_toggle):
        self.set_outcome_filter('xfail', tb_toggle.get_active())

    def on_test_treeview_row_activated(self, treeview, path, view_column):
        self._show_results_for_tree_path(path)
        
    def on_test_concluded(self, test_id, outcome):
        # called back by TribunalTestResult when a test finishes
        self.statusbar.pop(self._running_test_statusbar_context)
        self._update_outcome_counts()

    def on_test_started(self, test_id):
        self.statusbar.push(self._running_test_statusbar_context,
            "running %s" % test_id)
        pass
        
    def on_test_treeview_cursor_changed(self, treeview):
        path, column = treeview.get_cursor()
        # print 'cursor to path %r, column %r' % (path, column)
        self._show_results_for_tree_path(path)

    def _row_visible_func(self, model, iter, user_data=None):
        """Called back from TreeModelFilter to check if a row is visible."""
        row_outcome = model.get_value(iter, tree.TableTestResults.COL_OUTCOME)
        return self._outcome_filter.get(row_outcome, True)

    def set_outcome_filter(self, outcome, show):
        if show != self._outcome_filter[outcome]:
            self._outcome_filter[outcome] = show
            self._get_widget('show_%s_tb' % outcome).set_active(show)
            self._get_widget('filter_%s_menuitem' % outcome).set_active(show)
            self.refresh_filter()
            
    def set_source(self, test_source):
        self.source = test_source

    def show_all(self):
        # show the top level window and the rest will follow
        self.subunit_window.show_all()
    
    def _show_results_for_tree_path(self, path):
        # apparent path is given in the filtered view; the actual path
        # we need is the one in the underlying model
        real_path = self.model_filter.convert_path_to_child_path(path)
        test_id = self.test_result.get_test_id(real_path)
        if test_id:
            details = self.test_result.get_detail_text_by_test_id(
                    test_id)
            outcome = self.test_result.get_outcome_by_test_id(test_id)
            error = self.test_result.get_error_by_test_id(test_id)
        else:
            error = details = outcome = None
        if test_id:
            self.result_frame_label.set_label('<b>%s %s:</b>' 
                % tuple(map(xml_escape, [test_id, outcome])))
            self.result_frame.set_sensitive(True)
        else:
            self.result_frame_label.set_label('Details:')
            self.result_frame.set_sensitive(False)
        if details or error:
            self.result_textview.set_sensitive(True)
            t = (error or '') + '\n' + details_dict_to_text(details)
            self.result_textview.get_buffer().set_text(t)
        else:
            self.result_textview.set_sensitive(False)
            self.result_textview.get_buffer().set_text('')

    def refresh_filter(self):
        self.model_filter.refilter()

    def reload_from_source(self):
        self.load_from_subunit_stream(self.source.reload())
        
    def _update_outcome_counts(self):
        for status in ['failed', 'skipped', 'passed', 'xfail']:
            tb_button = self._glade_xml.get_widget('show_%s_tb' % status)
            tb_button.set_label("%d %s" %
                (self.test_result._counts_by_status[status],
                    status.capitalize()))



